@@ -1,48 +1,61 @@
-import requests
-import yt_dlp
-from typing import Dict, Any, List
+from typing import Dict, Any, List, Union, Tuple
 from .gpt import seg_transcript, chat_gpt
 from datetime import datetime
+from youtube_transcript_api import YouTubeTranscriptApi
 import traceback
 import re
 
 headers = {
-    'authority': 'api.youtube.com',
-    'accept': 'application/json, text/plain, */*',
-    'user-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/58.0.3029.110 Safari/537.3',
-    'accept-Language': 'zh-CN,zh;q=0.9,en;q=0.8,en-GB;q=0.7,en-US;q=0.6',
+    "authority": "api.youtube.com",
+    "accept": "application/json, text/plain, */*",
+    "user-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/58.0.3029.110 Safari/537.3",
+    "accept-Language": "zh-CN,zh;q=0.9,en;q=0.8,en-GB;q=0.7,en-US;q=0.6",
 }
 
+
 class HytGpt:
     def __init__(self, gpt_key: str, prompt: str):
         self.gpt_key = gpt_key
         self.prompt = prompt
-        
+
     def summary(self, ylink: str) -> Dict[str, str]:
         if not self.is_valid_youtube_url(ylink):
-            return {'status': 'failed', 'url': ylink, 'subtitles': [], 'summaries': 'Invalid youtube url'}
-        subtitles = self.__youtube_subtitle(ylink)
+            return {
+                "status": "failed",
+                "url": ylink,
+                "subtitles": [],
+                "summaries": "Invalid youtube url",
+            }
+        subtitles, language = self.__youtube_subtitle(ylink)
         if not subtitles:
-            return {'status': 'failed', 'url': ylink, 'subtitles': subtitles, 'summaries': 'Subtitle retrieval failed'}
+            return {
+                "status": "failed",
+                "url": ylink,
+                "subtitles": subtitles,
+                "summaries": "Subtitle retrieval failed",
+            }
 
         seged_text = seg_transcript(subtitles)
-        summaried_text = ''
+        summaried_text = ""
         i = 1
+
         for entry in seged_text:
             try:
-                response = chat_gpt(self.gpt_key, self.prompt, entry)
-                print(f'Completed the {str(i)} part summary')
+                response = chat_gpt(
+                    self.gpt_key, self.prompt.format(language=language), entry
+                )
+                print(f"Completed the {str(i)} part summary")
                 i += 1
             except Exception as e:
                 print(f"Exception occurred: {str(e)}")
                 traceback.print_exc()
-                response = 'Summary failed'
-            summaried_text += response + '\n'
+                response = "Summary failed"
+            summaried_text += response + "\n"
         response_data = {
-            'status': 'success',
-            'url': ylink,
-            'subtitles': subtitles,
-            'summaries': summaried_text,
+            "status": "success",
+            "url": ylink,
+            "subtitles": subtitles,
+            "summaries": summaried_text,
         }
         return response_data
 
@@ -50,32 +63,48 @@ class HytGpt:
     def is_valid_youtube_url(url):
         # Regular expression pattern for YouTube URLs
         youtube_url_pattern = re.compile(
-            r'(https?://)?(www\.)?(youtube\.com|youtu\.?be)/.+'
+            r"(https?://)?(www\.)?(youtube\.com|youtu\.?be)/.+"
         )
-        
+
         # Check if the URL matches the pattern
         if youtube_url_pattern.match(url):
             return True
         else:
             return False
-        
+
     @staticmethod
     def get_youtube_id(url: str) -> str:
-        youtube_id_match = re.search(r'(?<=v=)[^&#]+', url)
-        youtube_id_match = youtube_id_match or re.search(r'(?<=be/)[^&#]+', url)
+        youtube_id_match = re.search(r"(?<=v=)[^&#]+", url)
+        youtube_id_match = youtube_id_match or re.search(r"(?<=be/)[^&#]+", url)
         return youtube_id_match.group(0) if youtube_id_match else None
-    
+
     @staticmethod
     def build_youtube_url(youtube_id: str) -> str:
-        return f'https://www.youtube.com/watch?v={youtube_id}'
-    
+        return f"https://www.youtube.com/watch?v={youtube_id}"
+
     @staticmethod
     def get_timestamp_diff(start_time, end_time):
-        start_time = datetime.strptime(start_time, '%H:%M:%S.%f')
-        end_time = datetime.strptime(end_time, '%H:%M:%S.%f')
+        start_time = datetime.strptime(start_time, "%H:%M:%S.%f")
+        end_time = datetime.strptime(end_time, "%H:%M:%S.%f")
         diff = (end_time - start_time).total_seconds()
         return diff
 
+    def __youtube_subtitle(
+        self, url: str
+    ) -> Tuple[Union[List[Dict[str, str]], None], str]:
+        video_id = self.get_youtube_id(url)
+        list = YouTubeTranscriptApi.list_transcripts(video_id)
+        for transcript in list:
+            if transcript.language_code in ["en", "zh-Hans", "zh-Hant"]:
+                return transcript.fetch(), transcript.language
+        # if 'zh-Hans' in transicript.translation_languages:
+        #     translated = transicript.translate('zh-Hans').fetch()
+        #     return translated
+        return None
+
+
+"""
+5/5/23: Older version
 
     def __youtube_player_list(self, yvid):
         url = f"https://www.youtube.com/watch?v={yvid}"
@@ -151,7 +180,7 @@ class HytGpt:
         if 'requested_subtitles' in result:
             print('Found requested subtitles.')
             requested_subtitles = result.get('requested_subtitles')
-            if 'en' in requested_subtitles:
+            if isinstance(requested_subtitles, dict) and 'en' in requested_subtitles:
                 subtitle = requested_subtitles['en']
                 if subtitle.get('ext') == 'vtt':
                     print('No subtitles found, fetching requested subtitles.')
@@ -167,3 +196,4 @@ class HytGpt:
             if 'text' in subtitle:
                 results.append(subtitle.get('text'))
         return results
+"""
